package com.qmth.teachcloud.common.aspect;

import com.alibaba.fastjson.JSON;
import com.qmth.teachcloud.common.annotation.OperationLogDetail;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicOperationLog;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.util.ServletUtil;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.*;
import org.aspectj.lang.reflect.MethodSignature;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @Description: 日志操作记录拦截aop
 * @Author: CaoZixuan
 * @Date: 2022-02-14
 */
@Aspect
@Component
public class LogAspect {

    private final static Logger log = LoggerFactory.getLogger(LogAspect.class);

    /**
     * 日志注解切入点
     */
    @Pointcut("@annotation(com.qmth.teachcloud.common.annotation.OperationLogDetail)")
    public void operationLog(){

    }

    /**
     * 环绕增强，操作日志环绕切入
     * @param joinPoint joinPoint
     * @return return
     * @throws Throwable Throwable
     */
    @Around(value = "operationLog()")
    public Object aroundOperationLogPoint(ProceedingJoinPoint joinPoint) throws Throwable {
        Object res = null;
        long time = System.currentTimeMillis();
        try {
            res =  joinPoint.proceed();
            time = System.currentTimeMillis() - time;
            return res;
        } finally {
            try {
                //方法执行完成后增加日志
                addOperationLog(joinPoint,res,time);
            }catch (Exception e){
                System.out.println("LogAspect 操作失败：" + e.getMessage());
                e.printStackTrace();
            }
        }
    }

    /**
     * 记录日志操作
     * @param joinPoint joinPoint
     * @param res 返回值
     * @param time 方法执行时间
     */
    private void addOperationLog(JoinPoint joinPoint, Object res, long time){
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        String userName = requestUser.getRealName();

        MethodSignature signature = (MethodSignature)joinPoint.getSignature();
        BasicOperationLog operationLog = new BasicOperationLog();

        // 方法中的信息
        operationLog.setId(SystemConstant.getDbUuid());
        operationLog.setMethod(signature.getDeclaringTypeName() + "." + signature.getName());
        operationLog.setArgs(JSON.toJSONString(joinPoint.getArgs()));
        operationLog.setRunTime(time);
        operationLog.setReturnValue(JSON.toJSONString(res));
        operationLog.setUserId(requestUser.getId());
        operationLog.setUserName(userName);
        operationLog.setCreateTime(new Date().getTime());

        // 注解中的信息
        OperationLogDetail annotation = signature.getMethod().getAnnotation(OperationLogDetail.class);
        if(annotation != null){
            operationLog.setDetail(getDetail(((MethodSignature)joinPoint.getSignature()).getParameterNames(),joinPoint.getArgs(),annotation));
            operationLog.setLevel(annotation.level());
            operationLog.setOperationType(annotation.operationType());
            operationLog.setOperationUnit(annotation.operationUnit());
            operationLog.setSystemType(annotation.system());
        }
        //TODO 这里保存日志
        System.out.println(JSON.toJSONString(operationLog));
    }

    /**
     * 对当前登录用户和占位符处理
     * @param argNames 方法参数名称数组
     * @param args 方法参数数组
     * @param annotation 注解信息
     * @return 返回处理后的描述
     */
    private String getDetail(String[] argNames, Object[] args, OperationLogDetail annotation){
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();

        Map<Object, Object> map = new HashMap<>();
        for(int i = 0;i < argNames.length;i++){
            map.put(argNames[i],args[i]);
        }

        String detail = annotation.detail();
        try {
            detail = "'" + requestUser.getRealName() + "'=》" + annotation.detail();
            for (Map.Entry<Object, Object> entry : map.entrySet()) {
                Object k = entry.getKey();
                Object v = entry.getValue();
                detail = detail.replace("{{" + k + "}}", JSON.toJSONString(v));
            }
        }catch (Exception e){
            e.printStackTrace();
        }
        return detail;
    }

    @Before(value = "operationLog()")
    public void doBeforeAdvice(JoinPoint joinPoint){
        System.out.println("进入方法前执行.....");

    }

    /**
     * 处理完请求，返回内容
     * @param ret ret
     */
    @AfterReturning(returning = "ret", pointcut = "operationLog()")
    public void doAfterReturning(Object ret) {
        System.out.println("方法的返回值 : " + ret);
    }

    /**
     * 后置异常通知
     */
    @AfterThrowing(value = "operationLog()")
    public void throwss(JoinPoint jp){
        System.out.println("方法异常时执行.....");
    }


    /**
     * 后置最终通知,final增强，不管是抛出异常或者正常退出都会执行
     */
    @After(value = "operationLog()")
    public void after(JoinPoint jp){
        System.out.println("方法最后执行.....");
    }
}
